@@ -59,7 +59,7 @@ PreferencesDialog::PreferencesDialog(QWidget *parent) : QDialog(parent) {
 	tabs->addTab(w2, tr("Numbers && Operators"));
 	tabs->addTab(w3, tr("Units && Currencies"));
 	tabs->addTab(w4, tr("Parsing && Calculation"));
-	QCheckBox *box; QGridLayout *l; QComboBox *combo;
+	QAbstractButton *box; QGridLayout *l; QComboBox *combo;
 	int r = 0;
 	l = new QGridLayout(w1); l->setSizeConstraint(QLayout::SetFixedSize);
 	BOX(tr("Ignore system language (requires restart)"), settings->ignore_locale, ignoreLocaleToggled(bool));
@@ -103,10 +103,20 @@ PreferencesDialog::PreferencesDialog(QWidget *parent) : QDialog(parent) {
 #endif
 	BOX(tr("Allow multiple instances"), settings->allow_multiple_instances > 0, multipleInstancesToggled(bool));
 	BOX(tr("Clear history on exit"), settings->clear_history_on_exit, clearHistoryToggled(bool));
+	l->addWidget(new QLabel(tr("Max history lines saved:"), this), r, 0); QSpinBox *spin = new QSpinBox(this); spin->setRange(0, 100000); spin->setValue(settings->max_history_lines); connect(spin, SIGNAL(valueChanged(int)), this, SLOT(maxHistoryLinesChanged(int))); l->addWidget(spin, r, 1); r++;
 	BOX(tr("Close application with Escape key"), settings->close_with_esc, closeWithEscToggled(bool));
 	BOX(tr("Use keyboard keys for RPN"), settings->rpn_keys, rpnKeysToggled(bool));
 	BOX(tr("Use caret for bitwise XOR"), settings->caret_as_xor, caretAsXorToggled(bool));
 	BOX(tr("Keep above other windows"), settings->always_on_top, keepAboveToggled(bool));
+	QCheckBox *box2 = new QCheckBox(tr("Preserve history height"), this); l->addWidget(box2, r, 0, 1, 2); connect(box2, SIGNAL(stateChanged(int)), this, SLOT(preserveHeightChanged(int))); r++;
+	preserveHeightBox = box2;
+	if(settings->preserve_history_height < 0) {
+		box2->setTristate(true);
+		box2->setCheckState(Qt::PartiallyChecked);
+	} else {
+		box2->setChecked(settings->preserve_history_height > 0);
+	}
+	box2->setToolTip(tr("Do not change the height of history list when keypad or number bases are show or hidden."));
 	l->addWidget(new QLabel(tr("Window title:"), this), r, 0);
 	combo = new QComboBox(this);
 	combo->addItem(tr("Application name"), TITLE_APP);
@@ -155,20 +165,37 @@ PreferencesDialog::PreferencesDialog(QWidget *parent) : QDialog(parent) {
 	l->setRowStretch(r, 1);
 	r = 0;
 	l = new QGridLayout(w4); l->setSizeConstraint(QLayout::SetFixedSize);
-	box = new QCheckBox(tr("Display expression status"), this); l->addWidget(box, r, 0); box->setChecked(settings->display_expression_status); connect(box, SIGNAL(toggled(bool)), this, SLOT(expressionStatusToggled(bool)));
-	statusBox = box;
-	QHBoxLayout *hbox = new QHBoxLayout();
-	l->addLayout(hbox, r, 1); r++;
-	hbox->addWidget(new QLabel(tr("Delay:")));
+	l->addWidget(new QLabel(tr("Expression status:"), this), r, 0);
+	combo = new QComboBox(this);
+	combo->addItem(tr("Off"), 0);
+	combo->addItem(tr("In history list"), 1);
+	combo->addItem(tr("In expression field"), 2);
+	if(!settings->display_expression_status) combo->setCurrentIndex(0);
+	else if(settings->status_in_history) combo->setCurrentIndex(1);
+	else combo->setCurrentIndex(2);
+	statusCombo = combo;
+	connect(combo, SIGNAL(currentIndexChanged(int)), this, SLOT(statusModeChanged(int)));
+	l->addWidget(combo, r, 1); r++;
+	l->addWidget(new QLabel(tr("Status tooltip delay:")), r, 0);
 	statusDelayWidget = new QSpinBox(this);
 	statusDelayWidget->setRange(0, 10000);
 	statusDelayWidget->setSingleStep(250);
 	//: milliseconds
 	statusDelayWidget->setSuffix(" " + tr("ms"));
 	statusDelayWidget->setValue(settings->expression_status_delay); 
+	statusDelayWidget->setEnabled(settings->display_expression_status);
 	connect(statusDelayWidget, SIGNAL(valueChanged(int)), this, SLOT(statusDelayChanged(int)));
-	hbox->addWidget(statusDelayWidget);
-	hbox->addStretch(1);
+	l->addWidget(statusDelayWidget, r, 1); r++;
+	l->addWidget(new QLabel(tr("Calculate as you type delay:")), r, 0);
+	calculateDelayWidget = new QSpinBox(this);
+	calculateDelayWidget->setRange(0, 10000);
+	calculateDelayWidget->setSingleStep(250);
+	//: milliseconds
+	calculateDelayWidget->setSuffix(" " + tr("ms"));
+	calculateDelayWidget->setValue(settings->auto_calculate_delay);
+	calculateDelayWidget->setEnabled(settings->status_in_history);
+	connect(calculateDelayWidget, SIGNAL(valueChanged(int)), this, SLOT(calculateDelayChanged(int)));
+	l->addWidget(calculateDelayWidget, r, 1); r++;
 	l->addWidget(new QLabel(tr("Expression in history:"), this), r, 0);
 	combo = new QComboBox(this);
 	combo->addItem(tr("Parsed"), 0);
@@ -347,7 +374,7 @@ PreferencesDialog::PreferencesDialog(QWidget *parent) : QDialog(parent) {
 	connect(combo, SIGNAL(currentIndexChanged(int)), this, SLOT(temperatureCalculationChanged(int)));
 	box = new QCheckBox(tr("Exchange rates updates:"), this); box->setChecked(settings->auto_update_exchange_rates > 0); connect(box, SIGNAL(toggled(bool)), this, SLOT(exratesToggled(bool))); l->addWidget(box, r, 0);
 	int days = settings->auto_update_exchange_rates <= 0 ? 7 : settings->auto_update_exchange_rates;
-	QSpinBox *spin = new QSpinBox(this); spin->setRange(1, 100); spin->setValue(days); spin->setEnabled(settings->auto_update_exchange_rates > 0); connect(spin, SIGNAL(valueChanged(int)), this, SLOT(exratesChanged(int))); l->addWidget(spin, r, 1); exratesSpin = spin; r++;
+	spin = new QSpinBox(this); spin->setRange(1, 100); spin->setValue(days); spin->setEnabled(settings->auto_update_exchange_rates > 0); connect(spin, SIGNAL(valueChanged(int)), this, SLOT(exratesChanged(int))); l->addWidget(spin, r, 1); exratesSpin = spin; r++;
 	QString str = tr("%n day(s)", "", days);
 	int index = str.indexOf(QString::number(days));
 	if(index == 0) {
@@ -402,6 +429,15 @@ void PreferencesDialog::multipleInstancesToggled(bool b) {
 void PreferencesDialog::clearHistoryToggled(bool b) {
 	settings->clear_history_on_exit = b;
 }
+void PreferencesDialog::maxHistoryLinesChanged(int i) {
+	settings->max_history_lines = i;
+}
+void PreferencesDialog::preserveHeightChanged(int state) {
+	preserveHeightBox->setTristate(false);
+	settings->preserve_history_height = (state == Qt::Checked);
+	settings->keypad_appended = false;
+	settings->bases_appended = false;
+}
 void PreferencesDialog::keepAboveToggled(bool b) {
 	settings->always_on_top = b;
 	if(settings->always_on_top) setWindowFlags(windowFlags() | Qt::WindowStaysOnTopHint);
@@ -413,14 +449,19 @@ void PreferencesDialog::tooltipsChanged(int i) {
 	settings->enable_tooltips = qobject_cast<QComboBox*>(sender())->itemData(i).toInt();
 	emit enableTooltipsChanged();
 }
-void PreferencesDialog::expressionStatusToggled(bool b) {
-	settings->display_expression_status = b;
-	statusDelayWidget->setEnabled(b);
-	if(!b) QToolTip::hideText();
+void PreferencesDialog::statusModeChanged(int i) {
+	settings->display_expression_status = (i > 0);
+	settings->status_in_history = (i == 1);
+	statusDelayWidget->setEnabled(settings->display_expression_status);
+	calculateDelayWidget->setEnabled(settings->status_in_history);
+	emit statusModeChanged();
 }
 void PreferencesDialog::statusDelayChanged(int v) {
 	settings->expression_status_delay = v;
 }
+void PreferencesDialog::calculateDelayChanged(int v) {
+	settings->auto_calculate_delay = v;
+}
 void PreferencesDialog::binTwosToggled(bool b) {
 	settings->printops.twos_complement = b;
 	emit resultDisplayUpdated();
@@ -826,12 +867,18 @@ void PreferencesDialog::updateParsingMode() {
 }
 void PreferencesDialog::updateExpressionStatus() {
 	statusDelayWidget->blockSignals(true);
-	statusBox->blockSignals(true);
+	calculateDelayWidget->blockSignals(true);
+	statusCombo->blockSignals(true);
+	if(!settings->display_expression_status) statusCombo->setCurrentIndex(0);
+	else if(settings->status_in_history) statusCombo->setCurrentIndex(1);
+	else statusCombo->setCurrentIndex(2);
 	statusDelayWidget->setValue(settings->expression_status_delay);
-	statusBox->setChecked(settings->display_expression_status);
 	statusDelayWidget->setEnabled(settings->display_expression_status);
+	calculateDelayWidget->setValue(settings->auto_calculate_delay);
+	calculateDelayWidget->setEnabled(settings->status_in_history);
 	statusDelayWidget->blockSignals(false);
-	statusBox->blockSignals(false);
+	calculateDelayWidget->blockSignals(false);
+	statusCombo->blockSignals(false);
 }
 void PreferencesDialog::updateTemperatureCalculation() {
 	tcCombo->blockSignals(true);
